@@ -67,7 +67,6 @@ impl<UserData: 'static + WorldQuery + Send + Sync> Plugin for RapierPhysicsPlugi
         .insert_resource(IslandManager::new())
         .insert_resource(JointSet::new())
         .insert_resource(CCDSolver::new())
-        .insert_resource(PhysicsHooksWithQueryObject::<UserData>(Box::new(())))
         .insert_resource(Events::<IntersectionEvent>::default())
         .insert_resource(Events::<ContactEvent>::default())
         .insert_resource(SimulationToRenderTime::default())
@@ -75,24 +74,46 @@ impl<UserData: 'static + WorldQuery + Send + Sync> Plugin for RapierPhysicsPlugi
         .insert_resource(ModificationTracker::default())
         .add_system_to_stage(
             PhysicsStages::FinalizeCreations,
-            physics::attach_bodies_and_colliders_system.system(),
+            physics::attach_bodies_and_colliders_system
+                .system()
+                .label(physics::PhysicsSystems::AttachBodiesAndColliders),
         )
         .add_system_to_stage(
             PhysicsStages::FinalizeCreations,
-            physics::create_joints_system.system(),
+            physics::create_joints_system
+                .system()
+                .label(physics::PhysicsSystems::CreateJoints),
         )
         .add_system_to_stage(
             CoreStage::PreUpdate,
-            physics::finalize_collider_attach_to_bodies.system(),
+            physics::finalize_collider_attach_to_bodies
+                .system()
+                .label(physics::PhysicsSystems::FinalizeColliderAttachToBodies),
         )
         .add_system_to_stage(
             CoreStage::Update,
-            physics::step_world_system::<UserData>.system(),
+            physics::step_world_system::<UserData>
+                .system()
+                .label(physics::PhysicsSystems::StepWorld),
         )
         .add_system_to_stage(
             PhysicsStages::SyncTransforms,
-            physics::sync_transforms.system(),
+            physics::sync_transforms
+                .system()
+                .label(physics::PhysicsSystems::SyncTransforms),
         )
-        .add_system_to_stage(CoreStage::PostUpdate, physics::collect_removals.system());
+        .add_system_to_stage(
+            CoreStage::PostUpdate,
+            physics::collect_removals
+                .system()
+                .label(physics::PhysicsSystems::CollectRemovals),
+        );
+        if app
+            .world()
+            .get_resource::<PhysicsHooksWithQueryObject<UserData>>()
+            .is_none()
+        {
+            app.insert_resource(PhysicsHooksWithQueryObject::<UserData>(Box::new(())));
+        }
     }
 }
